﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using Random = UnityEngine.Random;

namespace Mod
{
    public class Mod : MonoBehaviour
    {
        public static string ModTag = " [SP]"; //Whilst not required, modded items should contain 'tags' at the end of their names to prevent errors in which two mods have an item of the same name.
        public static Transform portalBlue, portalOrange;
        public static Sprite spritePortalBlue = ModAPI.LoadSprite("portalblue.png");
        public static Sprite spritePortalOrange = ModAPI.LoadSprite("portalorange.png");
        public static AudioClip shootBlue = ModAPI.LoadSound("fireblue.mp3");
        public static AudioClip shootOrange = ModAPI.LoadSound("fireorange.mp3");
        public static AudioClip travel = ModAPI.LoadSound("travel.mp3");
        public static AudioClip gunkill = ModAPI.LoadSound("portalkill.mp3");
        public static void Main()
        {
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Handheld Portal Device" + ModTag,
                DescriptionOverride = "The Aperture Science Handheld Portal Device (also known as the Portal Gun, ASHPoD, Aperture Science Quantum Tunneling Device, or ASHPD) is the experimental tool designed to place two portals that objects can pass through. The Handheld Portal Device is designed to create and place portals on immobile surfaces, or surfaces covered with Conversion Gel that are big enough for the Portals to be opened. When the device is fired, a burst of energy will be released from the end of the device. If the projectile strikes a valid surface, a portal of the corresponding color will be formed. If a portal of the other color is already placed, these two portals will be linked. If a portal of the same color is already placed, it will be closed and the new one will be used instead, limiting the player to the use of only two portals at a time. Nothing will happen if a portal is shot at an enemy or other object. The glass chamber and a light on the top of the device will glow with the color of the previously placed portal.",
                CategoryOverride = ModAPI.FindCategory("Firearms"),
                ThumbnailOverride = ModAPI.LoadSprite("portalgunthumb.png"), //For convienence, it's best to place all sound effects and sprites within the sfx and sprites folder that comes with this base.
                AfterSpawn = (Instance) =>
                {
                    Instance.AddComponent<PortalGun>();
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("portalgun.png");
                    Instance.FixColliders();
                }
            });
        }
    }
    public class Portal : MonoBehaviour
    {
        Transform originalParent;
        [SkipSerialisation]
        public bool isBlue;
        Vector3 origLocalPos;
        //Rigidbody2D rb;
        void Start()
        {
            originalParent = transform.parent;
            BoxCollider2D col = gameObject.AddComponent<BoxCollider2D>();
            col.size = new Vector2(ModAPI.PixelSize * 50f, ModAPI.PixelSize * 15f);
            col.offset = new Vector2(0f, ModAPI.PixelSize * 7.5f);
            col.isTrigger = true;
            Physics2D.IgnoreCollision(col, originalParent.GetComponent<Collider2D>(), true);
            //rb = gameObject.AddComponent<Rigidbody2D>();
            col = gameObject.AddComponent<BoxCollider2D>();
            col.size = new Vector2(ModAPI.PixelSize * 50f, ModAPI.PixelSize * 20f);
            col.offset = new Vector2(-ModAPI.PixelSize * 50f, -ModAPI.PixelSize * 10f);
            Physics2D.IgnoreCollision(col, originalParent.GetComponent<Collider2D>(), true);
            col = gameObject.AddComponent<BoxCollider2D>();
            col.size = new Vector2(ModAPI.PixelSize * 50f, ModAPI.PixelSize * 20f);
            col.offset = new Vector2(ModAPI.PixelSize * 50f, -ModAPI.PixelSize * 10f);
            Physics2D.IgnoreCollision(col, originalParent.GetComponent<Collider2D>(), true);
            //rb.isKinematic = true;
            //rb.collisionDetectionMode = CollisionDetectionMode2D.Continuous;
            origLocalPos = transform.localPosition;
            GetComponent<SpriteRenderer>().material = ModAPI.FindMaterial("VeryBright");
        }
        public IEnumerator disableColShort(Collider2D col1, Collider2D col2)
        {
            Physics2D.IgnoreCollision(col1, col2, true);
            yield return new WaitForSeconds(0.4f);
            Physics2D.IgnoreCollision(col1, col2, false);
            yield return null;
        }
        void Update()
        {
            transform.parent = null;
            transform.localScale = new Vector3(Mathf.Lerp(transform.localScale.x, 1f, 8f * Time.deltaTime), 1f, 1f);
            transform.parent = originalParent;
            transform.localPosition = origLocalPos;
        }
        void OnTriggerEnter2D(Collider2D col)
        {
            if (col.transform != originalParent)
                if (col.attachedRigidbody)
                {
                    Physics2D.IgnoreCollision(col, originalParent.GetComponent<Collider2D>(), true);
                    //AudioSource.PlayClipAtPoint(Mod.travelIn, transform.position);
                }
        }
        void OnTriggerExit2D(Collider2D col)
        {
            if (col.transform != originalParent)
            {
                if (col.attachedRigidbody && !col.attachedRigidbody.isKinematic)
                {
                    var heading = col.transform.position - transform.position;
                    float dot = Vector3.Dot(heading, transform.up);
                    if (!col.GetComponent<LimbBehaviour>() || dot > 0f)
                        foreach (Collider2D cols in col.GetComponents<Collider2D>())
                        {
                            Physics2D.IgnoreCollision(cols, originalParent.GetComponent<Collider2D>(), false);
                        }

                    if (dot < 0f)
                    {
                        if (!col.GetComponent<LimbBehaviour>() || col.GetComponent<LimbBehaviour>().RoughClassification == LimbBehaviour.BodyPart.Head)
                        {
                            AudioSource.PlayClipAtPoint(Mod.travel, transform.position);
                            if (isBlue)
                            {
                                //print(Vector2.Dot)
                                List<Vector2> limbOffs = new List<Vector2>();
                                if (col.GetComponent<LimbBehaviour>())
                                {
                                    PersonBehaviour person = col.GetComponent<LimbBehaviour>().Person;
                                    person.PainLevel = 1f;
                                    person.ShockLevel = 0.6f;
                                    col.GetComponent<LimbBehaviour>().Joint.autoConfigureConnectedAnchor = false;
                                    foreach (LimbBehaviour limb in col.GetComponent<LimbBehaviour>().Person.Limbs)
                                    {
                                        if (limb.Joint)
                                            limb.Joint.autoConfigureConnectedAnchor = false;
                                        limbOffs.Add(limb.transform.position - col.transform.position);
                                    }
                                }
                                Transform origParent = col.transform.parent;
                                col.transform.parent = transform;
                                Vector3 locpos = col.transform.localPosition;
                                col.transform.parent = Mod.portalOrange;
                                col.transform.localPosition = locpos;
                                col.transform.parent = origParent;
                                col.transform.position = Vector3.MoveTowards(col.transform.position, Mod.portalOrange.position, ModAPI.PixelSize * 4f);
                                col.transform.eulerAngles += new Vector3(0f, 0f, Mod.portalOrange.eulerAngles.z - transform.eulerAngles.z);
                                //col.attachedRigidbody.velocity = (Vector2)Mod.portalOrange.up * (Vector2.Dot(col.attachedRigidbody.velocity.normalized, -transform.up) * col.attachedRigidbody.velocity) + Vector3.Project(col.attachedRigidbody.velocity.normalized, transform.right) * (new Vector2(Mathf.Abs(col.attachedRigidbody.velocity.x), Mathf.Abs(col.attachedRigidbody.velocity.y)) * transform.right);
                                col.attachedRigidbody.velocity = Mod.portalOrange.up * col.attachedRigidbody.velocity.magnitude;
                                if (col.GetComponent<LimbBehaviour>())
                                {
                                    int i = 0;
                                    foreach (Vector2 vec in limbOffs)
                                    {
                                        col.GetComponent<LimbBehaviour>().Person.Limbs[i].transform.position = (Vector2)col.transform.position + vec;
                                        col.GetComponent<LimbBehaviour>().Person.Limbs[i].GetComponent<Rigidbody2D>().velocity = col.attachedRigidbody.velocity;
                                        Physics2D.IgnoreCollision(col.GetComponent<LimbBehaviour>().Person.Limbs[i].Collider, originalParent.GetComponent<Collider2D>(), false);
                                        Physics2D.IgnoreCollision(col.GetComponent<LimbBehaviour>().Person.Limbs[i].Collider, Mod.portalOrange.parent.GetComponent<Collider2D>(), true);
                                        i++;
                                    }
                                    Collider2D[] cols = Mod.portalOrange.GetComponents<Collider2D>();
                                    foreach (LimbBehaviour limb in col.GetComponent<LimbBehaviour>().Person.Limbs)
                                    {
                                        StartCoroutine(disableColShort(limb.Collider, cols[1]));
                                        StartCoroutine(disableColShort(limb.Collider, cols[2]));
                                    }
                                }
                                Physics2D.IgnoreCollision(col, Mod.portalOrange.parent.GetComponent<Collider2D>(), true);
                            }
                            else
                            {
                                List<Vector2> limbOffs = new List<Vector2>();

                                if (col.GetComponent<LimbBehaviour>())
                                {
                                    PersonBehaviour person = col.GetComponent<LimbBehaviour>().Person;
                                    person.PainLevel = 1f;
                                    person.ShockLevel = 0.6f;
                                    col.GetComponent<LimbBehaviour>().Joint.autoConfigureConnectedAnchor = false;
                                    foreach (LimbBehaviour limb in col.GetComponent<LimbBehaviour>().Person.Limbs)
                                    {
                                        if (limb.Joint)
                                            limb.Joint.autoConfigureConnectedAnchor = false;
                                        limbOffs.Add(limb.transform.position - col.transform.position);
                                    }
                                }
                                Transform origParent = col.transform.parent;
                                col.transform.parent = transform;
                                Vector3 locpos = col.transform.localPosition;
                                col.transform.parent = Mod.portalBlue;
                                col.transform.localPosition = locpos;
                                col.transform.parent = origParent;
                                col.transform.position = Vector3.MoveTowards(col.transform.position, Mod.portalBlue.position, ModAPI.PixelSize * 4f);
                                col.transform.eulerAngles += new Vector3(0f, 0f, Mod.portalBlue.eulerAngles.z - transform.eulerAngles.z);
                                //col.attachedRigidbody.velocity = (Vector2)Mod.portalBlue.up * (Vector2.Dot(col.attachedRigidbody.velocity.normalized, -transform.up) * col.attachedRigidbody.velocity) + Vector3.Project(col.attachedRigidbody.velocity.normalized, transform.right) * (new Vector2(Mathf.Abs(col.attachedRigidbody.velocity.x), Mathf.Abs(col.attachedRigidbody.velocity.y)) * transform.right);
                                col.attachedRigidbody.velocity = Mod.portalBlue.up * col.attachedRigidbody.velocity.magnitude;
                                if (col.GetComponent<LimbBehaviour>())
                                {
                                    int i = 0;
                                    foreach (Vector2 vec in limbOffs)
                                    {
                                        col.GetComponent<LimbBehaviour>().Person.Limbs[i].transform.position = (Vector2)col.transform.position + vec;
                                        col.GetComponent<LimbBehaviour>().Person.Limbs[i].GetComponent<Rigidbody2D>().velocity = col.attachedRigidbody.velocity;
                                        Physics2D.IgnoreCollision(col.GetComponent<LimbBehaviour>().Person.Limbs[i].Collider, originalParent.GetComponent<Collider2D>(), false);
                                        Physics2D.IgnoreCollision(col.GetComponent<LimbBehaviour>().Person.Limbs[i].Collider, Mod.portalBlue.parent.GetComponent<Collider2D>(), true);
                                        i++;
                                    }
                                    Collider2D[] cols = Mod.portalBlue.GetComponents<Collider2D>();
                                    foreach (LimbBehaviour limb in col.GetComponent<LimbBehaviour>().Person.Limbs)
                                    {
                                        StartCoroutine(disableColShort(limb.Collider, cols[1]));
                                        StartCoroutine(disableColShort(limb.Collider, cols[2]));
                                    }
                                }
                                Physics2D.IgnoreCollision(col, Mod.portalBlue.parent.GetComponent<Collider2D>(), true);
                            }
                        }
                    }
                }
            }
        }
    }
    public class PortalGun : MonoBehaviour
    {
        LightSprite portalGlow;
        bool nextBlue;
        void Start()
        {
            nextBlue = true;
            portalGlow = ModAPI.CreateLight(transform, Color.blue, 2f, 1f);
            portalGlow.transform.localPosition = new Vector3(-ModAPI.PixelSize * 11f, ModAPI.PixelSize * 4f);
        }
        void Use(ActivationPropagation activation)
        {
            print((Vector2)transform.localScale);
            RaycastHit2D ray;
            if (transform.localScale.x > 0f)
                ray = Physics2D.Raycast((Vector2)transform.position + (((Vector2)transform.right) * ModAPI.PixelSize * 21f), (Vector2)transform.right, 1000f);
            else
            {
                ray = Physics2D.Raycast((Vector2)transform.position + ((-(Vector2)transform.right) * ModAPI.PixelSize * 21f), -(Vector2)transform.right, 1000f);
            }
            if (ray && ray.collider != GetComponent<Collider2D>())
            {
                if (ray.collider.GetComponent<LimbBehaviour>())
                {
                    GetComponent<PhysicalBehaviour>().PlayClipOnce(Mod.gunkill, 1f);
                    ModAPI.CreateParticleEffect("BigZap", ray.point);
                    foreach (LimbBehaviour limb in ray.collider.GetComponent<LimbBehaviour>().Person.Limbs)
                    {
                        limb.Slice();
                        limb.GetComponent<Rigidbody2D>().AddForce(new Vector2(Random.Range(-33f, 33f), Random.Range(-33f, 33f)), ForceMode2D.Impulse);
                    }
                }
                else
                {
                    GameObject g;
                    if (nextBlue)
                    {
                        if (Mod.portalBlue)
                            Destroy(Mod.portalBlue.gameObject);
                        g = new GameObject("PortalBlue", typeof(SpriteRenderer));
                        g.transform.localScale = new Vector3(0.1f, 1f, 1f);
                        ModAPI.CreateLight(g.transform, Color.cyan, 5f, 1f);
                        Mod.portalBlue = g.transform;
                        g.GetComponent<SpriteRenderer>().sprite = Mod.spritePortalBlue;
                        g.GetComponent<SpriteRenderer>().sortingLayerName = "Decals";
                        g.transform.position = ray.point;
                        g.transform.SetParent(ray.transform);
                        g.transform.rotation = Quaternion.LookRotation(Vector3.forward, ray.normal);
                        ModAPI.CreateParticleEffect("Spark", ray.point);
                        g.AddComponent<Portal>().isBlue = true;
                        portalGlow.Color = Color.yellow;
                        g.layer = 9;
                        g.AddComponent<Optout>();
                        GetComponent<PhysicalBehaviour>().PlayClipOnce(Mod.shootBlue, 1f);
                    }
                    else
                    {
                        if (Mod.portalOrange)
                            Destroy(Mod.portalOrange.gameObject);
                        g = new GameObject("PortalBlue", typeof(SpriteRenderer));
                        g.transform.localScale = new Vector3(0.1f, 1f, 1f);
                        ModAPI.CreateLight(g.transform, Color.yellow, 5f, 1f);
                        Mod.portalOrange = g.transform;
                        g.GetComponent<SpriteRenderer>().sprite = Mod.spritePortalOrange;
                        g.GetComponent<SpriteRenderer>().sortingLayerName = "Decals";
                        g.transform.position = ray.point;
                        g.transform.SetParent(ray.transform);
                        g.transform.rotation = Quaternion.LookRotation(Vector3.forward, ray.normal);
                        ModAPI.CreateParticleEffect("Spark", ray.point);
                        g.AddComponent<Portal>().isBlue = false;
                        portalGlow.Color = Color.blue;
                        g.layer = 9;
                        g.AddComponent<Optout>();
                        GetComponent<PhysicalBehaviour>().PlayClipOnce(Mod.shootOrange, 1f);
                    }
                    nextBlue = !nextBlue;
                }
            }
        }
    }
}